@@ -1,5 +1,7 @@
 // Copyright (C) 2022 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+// Qt-Security score:significant reason:default
+
 
 #include "qquick3dspecularglossymaterial_p.h"
 #include "qquick3dobject_p.h"
@@ -498,7 +500,7 @@ QT_BEGIN_NAMESPACE
 */
 
 /*!
-    \qmlproperty float SpecularGlossyMaterial::clearcoatAmount
+    \qmlproperty real SpecularGlossyMaterial::clearcoatAmount
 
     This property defines the intensity of the clearcoat layer.
 
@@ -528,7 +530,7 @@ QT_BEGIN_NAMESPACE
 */
 
 /*!
-    \qmlproperty float SpecularGlossyMaterial::clearcoatRoughnessAmount
+    \qmlproperty real SpecularGlossyMaterial::clearcoatRoughnessAmount
 
     This property defines the roughness of the clearcoat layer.
     The default value is \c 0.0
@@ -566,7 +568,7 @@ QT_BEGIN_NAMESPACE
 */
 
 /*!
-    \qmlproperty float SpecularGlossyMaterial::transmissionFactor
+    \qmlproperty real SpecularGlossyMaterial::transmissionFactor
 
     This property defines the percentage of light that is transmitted through
     the material's surface.
@@ -598,7 +600,7 @@ QT_BEGIN_NAMESPACE
 */
 
 /*!
-    \qmlproperty float SpecularGlossyMaterial::thicknessFactor
+    \qmlproperty real SpecularGlossyMaterial::thicknessFactor
 
     This property defines the thickness of the volume beneath the surface.
     Unlike many other properties of SpecularGlossyMaterial, the value in defined
@@ -632,7 +634,7 @@ QT_BEGIN_NAMESPACE
 */
 
 /*!
-    \qmlproperty float SpecularGlossyMaterial::attenuationDistance
+    \qmlproperty real SpecularGlossyMaterial::attenuationDistance
 
     This property defines the Density of the medium given as the average
     distance that light travels in the medium before interacting with a
@@ -1386,7 +1388,7 @@ QSSGRenderGraphObject *QQuick3DSpecularGlossyMaterial::updateSpatialNode(QSSGRen
 
     QSSGRenderDefaultMaterial *material = static_cast<QSSGRenderDefaultMaterial *>(node);
 
-    material->specularModel = QSSGRenderDefaultMaterial::MaterialSpecularModel::KGGX;
+    material->specularModel = QSSGRenderDefaultMaterial::MaterialSpecularModel::SchlickGGX;
 
     if (m_dirtyAttributes & LightingModeDirty)
         material->lighting = QSSGRenderDefaultMaterial::MaterialLighting(m_lighting);
