@@ -1,31 +1,7 @@
-/****************************************************************************
-**
-** Copyright (C) 2021 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Quick 3D.
-**
-** $QT_BEGIN_LICENSE:GPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 or (at your option) any later version
-** approved by the KDE Free Qt Foundation. The licenses are as published by
-** the Free Software Foundation and appearing in the file LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+// Qt-Security score:significant reason:default
+
 
 #ifndef QQUICK3DPARTICLEEMITTER_H
 #define QQUICK3DPARTICLEEMITTER_H
@@ -51,6 +27,8 @@
 
 QT_BEGIN_NAMESPACE
 
+class QQuaternion;
+
 class Q_QUICK3DPARTICLES_EXPORT QQuick3DParticleEmitter : public QQuick3DNode
 {
     Q_OBJECT
@@ -72,11 +50,17 @@ class Q_QUICK3DPARTICLES_EXPORT QQuick3DParticleEmitter : public QQuick3DNode
     Q_PROPERTY(QVector3D particleRotationVelocity READ particleRotationVelocity WRITE setParticleRotationVelocity NOTIFY particleRotationVelocityChanged)
     Q_PROPERTY(QVector3D particleRotationVelocityVariation READ particleRotationVelocityVariation WRITE setParticleRotationVelocityVariation NOTIFY particleRotationVariationVelocityChanged)
     Q_PROPERTY(float depthBias READ depthBias WRITE setDepthBias NOTIFY depthBiasChanged)
+    Q_PROPERTY(bool reversed READ reversed WRITE setReversed NOTIFY reversedChanged REVISION(6, 10))
 
+    Q_PROPERTY(EmitMode emitMode READ emitMode WRITE setEmitMode NOTIFY emitModeChanged REVISION(6, 10))
     QML_NAMED_ELEMENT(ParticleEmitter3D)
     QML_ADDED_IN_VERSION(6, 2)
 
 public:
+
+    enum EmitMode { Default = 0, SurfaceNormal, SurfaceReflected };
+    Q_ENUM(EmitMode)
+
     QQuick3DParticleEmitter(QQuick3DNode *parent = nullptr);
     ~QQuick3DParticleEmitter() override;
 
@@ -97,7 +81,9 @@ public:
     QVector3D particleRotationVelocity() const;
     QVector3D particleRotationVelocityVariation() const;
     float depthBias() const;
+    Q_REVISION(6, 10) bool reversed() const;
 
+    Q_REVISION(6, 10) EmitMode emitMode() const;
     QQmlListProperty<QQuick3DParticleEmitBurst> emitBursts();
 
     Q_INVOKABLE virtual void burst(int count);
@@ -122,7 +108,9 @@ public Q_SLOTS:
     void setParticleRotationVelocity(const QVector3D &particleRotationVelocity);
     void setParticleRotationVelocityVariation(const QVector3D &particleRotationVelocityVariation);
     void setDepthBias(float bias);
+    Q_REVISION(6, 10) void setReversed(bool reversed);
 
+    Q_REVISION(6, 10) void setEmitMode(EmitMode mode);
 Q_SIGNALS:
     void velocityChanged();
     void systemChanged();
@@ -141,6 +129,8 @@ Q_SIGNALS:
     void particleRotationVariationVelocityChanged();
     void enabledChanged();
     void depthBiasChanged();
+    Q_REVISION(6, 10) void reversedChanged();
+    Q_REVISION(6, 10) void emitModeChanged();
 
 protected:
     friend class QQuick3DParticleSystem;
@@ -151,7 +141,8 @@ protected:
     void registerEmitBurst(QQuick3DParticleEmitBurst *emitBurst);
     void unRegisterEmitBurst(QQuick3DParticleEmitBurst *emitBurst);
     void generateEmitBursts();
-    void emitParticle(QQuick3DParticle *particle, float startTime, const QMatrix4x4 &transform, const QQuaternion &parentRotation, const QVector3D &centerPos, int index = -1);
+    void emitParticle(QQuick3DParticle *particle, float startTime, const QMatrix4x4 &transform, const QQuaternion &parentRotation, const QVector3D &centerPos, int index = -1,
+                      const QVector3D &velocity = QVector3D(), const QVector3D &normal = QVector3D());
     void emitParticles();
     void emitActivationNodeParticles(QQuick3DParticleModelBlendParticle *particle);
     void emitParticlesBurst(const QQuick3DParticleEmitBurstData &burst);
@@ -213,6 +204,8 @@ private:
     // This list contains all emit bursts (both QQuick3DParticleEmitBurst and QQuick3DParticleDynamicBurst)
     QList<QQuick3DParticleEmitBurst *> m_emitBursts;
     QList<BurstEmitData> m_burstEmitData;
+    bool m_reversed = false;
+    EmitMode m_emitMode = EmitMode::Default;
 };
 
 QT_END_NAMESPACE
