@@ -1,6 +1,8 @@
 // Copyright (C) 2008-2012 NVIDIA Corporation.
 // Copyright (C) 2019 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+// Qt-Security score:significant reason:default
+
 
 
 #include "qssgrendergraphobject.h"
@@ -20,6 +22,7 @@ static const char *asString(QSSGRenderGraphObject::Type type)
         RETURN_AS_STRING(Type::Skeleton)
         RETURN_AS_STRING(Type::ImportScene)
         RETURN_AS_STRING(Type::ReflectionProbe)
+        RETURN_AS_STRING(Type::SceneRoot)
         RETURN_AS_STRING(Type::DirectionalLight)
         RETURN_AS_STRING(Type::PointLight)
         RETURN_AS_STRING(Type::SpotLight)
@@ -38,6 +41,7 @@ static const char *asString(QSSGRenderGraphObject::Type type)
         RETURN_AS_STRING(Type::ModelInstance)
         RETURN_AS_STRING(Type::ModelBlendParticle)
         RETURN_AS_STRING(Type::ResourceLoader)
+        RETURN_AS_STRING(Type::RenderPass)
         RETURN_AS_STRING(Type::DefaultMaterial)
         RETURN_AS_STRING(Type::PrincipledMaterial)
         RETURN_AS_STRING(Type::CustomMaterial)
@@ -46,6 +50,7 @@ static const char *asString(QSSGRenderGraphObject::Type type)
         RETURN_AS_STRING(Type::Image2D)
         RETURN_AS_STRING(Type::ImageCube)
         RETURN_AS_STRING(Type::RenderExtension)
+        RETURN_AS_STRING(Type::TextureProvider)
     }
 #undef RETURN_AS_STRING
     return nullptr;
