@@ -1,12 +1,14 @@
 // Copyright (C) 2008-2012 NVIDIA Corporation.
 // Copyright (C) 2019 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+// Qt-Security score:significant reason:default
+
 
 #include "qssgrendershadercodegenerator_p.h"
 
 #include <QtQuick3DUtils/private/qssgutils_p.h>
 
-#include <QtQuick3DRuntimeRender/private/qssgrendercontextcore_p.h>
+#include "qssgrendercontextcore.h"
 #include <QtQuick3DRuntimeRender/private/qssgrendershaderlibrarymanager_p.h>
 #include <QtQuick3DRuntimeRender/private/qssgshaderresourcemergecontext_p.h>
 
@@ -30,6 +32,8 @@ static inline void addEndCond(QByteArray &block, const T &var)
         block += QByteArrayLiteral("#endif\n");
 }
 
+int QSSGShaderResourceMergeContext::s_additionalBuffers = 0;
+
 struct QSSGShaderGeneratedProgramOutput
 {
     // never null; so safe to call strlen on.
@@ -45,7 +49,7 @@ struct QSSGShaderGeneratedProgramOutput
 
 QSSGStageGeneratorBase::QSSGStageGeneratorBase(QSSGShaderGeneratorStage inStage)
 
-    : m_outgoing(nullptr), m_stage(inStage)
+    : m_stage(inStage)
 {
 }
 
@@ -53,6 +57,8 @@ void QSSGStageGeneratorBase::begin(QSSGShaderGeneratorStageFlags inEnabledStages
 {
     m_incoming.clear();
     m_outgoing = nullptr;
+    m_flatIncoming.clear();
+    m_flatOutgoing = nullptr;
     m_includes.clear();
     m_uniforms.clear();
     m_uniformArrays.clear();
@@ -63,12 +69,15 @@ void QSSGStageGeneratorBase::begin(QSSGShaderGeneratorStageFlags inEnabledStages
     m_enabledStages = inEnabledStages;
     m_addedFunctions.clear();
     m_addedDefinitions.clear();
+    m_addedTypeDeclarations.clear();
+    m_prefixes.clear();
     // the shared buffers will be cleared elsewhere.
 }
 
 void QSSGStageGeneratorBase::addIncoming(const QByteArray &name, const QByteArray &type)
 {
     m_incoming.insert(name, type);
+    m_flatIncoming.remove(name);
 }
 
 void QSSGStageGeneratorBase::addOutgoing(const QByteArray &name, const QByteArray &type)
@@ -78,6 +87,25 @@ void QSSGStageGeneratorBase::addOutgoing(const QByteArray &name, const QByteArra
         return;
     }
     m_outgoing->insert(name, type);
+    if (m_flatOutgoing)
+        m_flatOutgoing->remove(name);
+}
+
+void QSSGStageGeneratorBase::addFlatIncoming(const QByteArray &name, const QByteArray &type)
+{
+    m_flatIncoming.insert(name, type);
+    m_incoming.remove(name);
+}
+
+void QSSGStageGeneratorBase::addFlatOutgoing(const QByteArray &name, const QByteArray &type)
+{
+    if (m_flatOutgoing == nullptr) {
+        Q_ASSERT(false);
+        return;
+    }
+    m_flatOutgoing->insert(name, type);
+    if (m_outgoing)
+        m_outgoing->remove(name);
 }
 
 void QSSGStageGeneratorBase::addUniform(const QByteArray &name, const QByteArray &type)
@@ -122,22 +150,22 @@ void QSSGStageGeneratorBase::addShaderPass2Marker(QSSGStageGeneratorBase::Shader
     m_finalBuilder.append(QByteArrayLiteral("//@@") + QByteArray::number(int(itemType)) + QByteArrayLiteral("\n"));
 }
 
-void QSSGStageGeneratorBase::addShaderItemMap(QSSGStageGeneratorBase::ShaderItemType itemType, const TStrTableStrMap &itemMap, const QByteArray &inItemSuffix)
+void QSSGStageGeneratorBase::addShaderItemMap(QSSGStageGeneratorBase::ShaderItemType itemType, const TStrTableStrMap &itemMap, ShaderItemMapFlags flags)
 {
     m_finalBuilder.append("\n");
 
     Q_ASSERT(m_mergeContext);
     for (TStrTableStrMap::const_iterator iter = itemMap.begin(), end = itemMap.end(); iter != end; ++iter) {
-        const QByteArray name = iter.key() + inItemSuffix;
+        const QByteArray name = iter.key();
         switch (itemType) {
         case ShaderItemType::VertexInput:
             m_mergeContext->registerInput(QSSGShaderGeneratorStage::Vertex, iter.value(), name);
             break;
         case ShaderItemType::Input:
-            m_mergeContext->registerInput(m_stage, iter.value(), name);
+            m_mergeContext->registerInput(m_stage, iter.value(), name, flags.testFlag(ShaderItemMapFlag::Flat));
             break;
         case ShaderItemType::Output:
-            m_mergeContext->registerOutput(m_stage, iter.value(), name);
+            m_mergeContext->registerOutput(m_stage, iter.value(), name, flags.testFlag(ShaderItemMapFlag::Flat));
             break;
         case ShaderItemType::Uniform:
             if (iter.value().startsWith(QByteArrayLiteral("sampler")))
@@ -170,12 +198,15 @@ void QSSGStageGeneratorBase::addShaderUniformMap()
             m_mergeContext->registerUniformMember(iter.value().second, name);
     }
     addShaderPass2Marker(ShaderItemType::Uniform);
+    addShaderPass2Marker(ShaderItemType::Image);
 }
 
 void QSSGStageGeneratorBase::addShaderOutgoingMap()
 {
     if (m_outgoing)
         addShaderItemMap(ShaderItemType::Output, *m_outgoing);
+    if (m_flatOutgoing)
+        addShaderItemMap(ShaderItemType::Output, *m_flatOutgoing, ShaderItemMapFlag::Flat);
 
     addShaderPass2Marker(ShaderItemType::Output);
 }
@@ -214,6 +245,14 @@ void QSSGStageGeneratorBase::addInclude(const QByteArray &name) { m_includes.ins
 void QSSGStageGeneratorBase::buildShaderSourcePass1(QSSGShaderResourceMergeContext *mergeContext)
 {
     m_mergeContext = mergeContext;
+
+    if (m_prefixes.size())
+        m_finalBuilder.append("\n");
+    for (const auto &prefix : std::as_const(m_prefixes)) {
+        m_finalBuilder.append(prefix);
+        m_finalBuilder.append("\n");
+    }
+
     addShaderIncomingMap();
     addShaderUniformMap();
     addShaderConstantBufferItemMap("uniform", m_constantBuffers, m_constantBufferParams);
@@ -232,6 +271,11 @@ void QSSGStageGeneratorBase::buildShaderSourcePass1(QSSGShaderResourceMergeConte
         m_finalBuilder.append("\n#endif\n");
     }
 
+    for (const auto& value : std::as_const(m_addedTypeDeclarations)) {
+        m_finalBuilder.append(value);
+        m_finalBuilder.append("\n");
+    }
+
     // Sort for deterministic shader text when printing/debugging
     QList<QByteArray> sortedIncludes(m_includes.begin(), m_includes.end());
     std::sort(sortedIncludes.begin(), sortedIncludes.end());
@@ -251,6 +295,9 @@ QByteArray QSSGStageGeneratorBase::buildShaderSourcePass2(QSSGShaderResourceMerg
     const int prefixLen = 4;
     const int typeLen = 1;
     int from = 0;
+
+    mergeContext->rearrangeResources();
+
     for (; ;) {
         int pos = m_finalBuilder.indexOf(prefix, from);
         if (pos >= 0) {
@@ -260,7 +307,7 @@ QByteArray QSSGStageGeneratorBase::buildShaderSourcePass2(QSSGShaderResourceMerg
             case ShaderItemType::VertexInput:
                 if (m_stage == QSSGShaderGeneratorStage::Vertex) {
                     QByteArray block;
-                    for (const QSSGShaderResourceMergeContext::InOutVar &var : mergeContext->m_inOutVars) {
+                    for (const QSSGShaderResourceMergeContext::InOutVar &var : std::as_const(mergeContext->m_inOutVars)) {
                         if (var.stagesInputIn.testFlag(m_stage))
                             block += QString::asprintf("layout(location = %d) in %s %s;\n", var.location, var.type.constData(), var.name.constData()).toUtf8();
                     }
@@ -270,9 +317,9 @@ QByteArray QSSGStageGeneratorBase::buildShaderSourcePass2(QSSGShaderResourceMerg
             case ShaderItemType::Input:
             {
                 QByteArray block;
-                for (const QSSGShaderResourceMergeContext::InOutVar &var : mergeContext->m_inOutVars) {
+                for (const QSSGShaderResourceMergeContext::InOutVar &var : std::as_const(mergeContext->m_inOutVars)) {
                     if (var.stagesInputIn.testFlag(m_stage))
-                        block += QString::asprintf("layout(location = %d) in %s %s;\n", var.location, var.type.constData(), var.name.constData()).toUtf8();
+                        block += QString::asprintf("layout(location = %d) in %s%s %s;\n", var.location, var.flat ? "flat " : "", var.type.constData(), var.name.constData()).toUtf8();
                 }
                 m_finalBuilder.replace(pos, prefixLen + typeLen, block);
             }
@@ -280,13 +327,29 @@ QByteArray QSSGStageGeneratorBase::buildShaderSourcePass2(QSSGShaderResourceMerg
             case ShaderItemType::Output:
             {
                 QByteArray block;
-                for (const QSSGShaderResourceMergeContext::InOutVar &var : mergeContext->m_inOutVars) {
+                for (const QSSGShaderResourceMergeContext::InOutVar &var : std::as_const(mergeContext->m_inOutVars)) {
                     if (var.stageOutputFrom.testFlag(m_stage))
-                        block += QString::asprintf("layout(location = %d) out %s %s;\n", var.location, var.type.constData(), var.name.constData()).toUtf8();
+                        block += QString::asprintf("layout(location = %d) out %s%s %s;\n", var.location, var.flat ? "flat " : "", var.type.constData(), var.name.constData()).toUtf8();
                 }
                 m_finalBuilder.replace(pos, prefixLen + typeLen, block);
             }
                 break;
+            case ShaderItemType::Image:
+            {
+                QByteArray block;
+
+                for (const auto &image : std::as_const(mergeContext->m_images)) {
+                    addStartCond(block, image);
+                    block += QString::asprintf("layout(binding = %d, %s) uniform %s %s %s;\n",
+                                               image.binding,
+                                               image.imgType.constData(),
+                                               image.qualifiers.constData(),
+                                               image.type.constData(),
+                                               image.name.constData()).toUtf8();
+                    addEndCond(block, image);
+                }
+                m_finalBuilder.replace(pos, prefixLen + typeLen, block);
+            } break;
             case ShaderItemType::Uniform:
             {
                 QByteArray block;
@@ -346,11 +409,23 @@ void QSSGStageGeneratorBase::addFunction(const QByteArray &functionName)
     }
 }
 
+void QSSGStageGeneratorBase::addPrefix(const QByteArray &data)
+{
+    m_prefixes.append(data);
+}
+
 void QSSGStageGeneratorBase::addDefinition(const QByteArray &name, const QByteArray &value)
 {
     m_addedDefinitions.insert(name, value);
 }
 
+void QSSGStageGeneratorBase::addTypeDeclaration(const QByteArray &typeName, const QByteArray &snippet)
+{
+    if (snippet.isEmpty())
+        return;
+    m_addedTypeDeclarations.insert(typeName, snippet);
+}
+
 void QSSGProgramGenerator::linkStages()
 {
     // Link stages incoming to outgoing variables.
@@ -361,8 +436,10 @@ void QSSGProgramGenerator::linkStages()
         QSSGShaderGeneratorStage theStageEnum = static_cast<QSSGShaderGeneratorStage>(theStageId);
         if ((m_enabledStages & theStageEnum)) {
             thisStage = &internalGetStage(theStageEnum);
-            if (previous)
+            if (previous) {
                 previous->m_outgoing = &thisStage->m_incoming;
+                previous->m_flatOutgoing = &thisStage->m_flatIncoming;
+            }
             previous = thisStage;
         }
     }
@@ -404,20 +481,34 @@ void QSSGProgramGenerator::registerShaderMetaDataFromSource(QSSGShaderResourceMe
     QSSGRenderShaderMetadata::ShaderMetaData meta = QSSGRenderShaderMetadata::getShaderMetaData(contents);
 
     for (const QSSGRenderShaderMetadata::Uniform &u : std::as_const(meta.uniforms)) {
-        if (u.type.startsWith(QByteArrayLiteral("sampler")))
-            mergeContext->registerSampler(u.type, u.name, u.condition, u.conditionName);
-        else
-            mergeContext->registerUniformMember(u.type, u.name, u.condition, u.conditionName);
+        if (u.type.startsWith(QByteArrayLiteral("sampler"))) {
+            if (u.multiview && mergeContext->viewCount >= 2) {
+                // 'sampler2D qt_screenTexture' becomes 'sampler2DArray qt_screenTextureArray'
+                mergeContext->registerSampler(u.type + QByteArrayLiteral("Array"), u.name + QByteArrayLiteral("Array"), u.condition, u.conditionName);
+            } else {
+                mergeContext->registerSampler(u.type, u.name, u.condition, u.conditionName);
+            }
+        } else {
+            if (u.multiview && mergeContext->viewCount >= 2) {
+                const QByteArray name = u.name + "[" + QByteArray::number(mergeContext->viewCount) + "]";
+                mergeContext->registerUniformMember(u.type, name, u.condition, u.conditionName);
+            } else {
+                mergeContext->registerUniformMember(u.type, u.name, u.condition, u.conditionName);
+            }
+        }
     }
 
+    for (const QSSGRenderShaderMetadata::Image &img : std::as_const(meta.images))
+        mergeContext->registerImage(img.type, img.name, img.imageType, img.qualifiers, img.condition, img.conditionName);
+
     for (const QSSGRenderShaderMetadata::InputOutput &inputVar : std::as_const(meta.inputs)) {
         if (inputVar.stage == stage)
-            mergeContext->registerInput(stage, inputVar.type, inputVar.name);
+            mergeContext->registerInput(stage, inputVar.type, inputVar.name, inputVar.flat);
     }
 
     for (const QSSGRenderShaderMetadata::InputOutput &outputVar : std::as_const(meta.outputs)) {
         if (outputVar.stage == stage)
-            mergeContext->registerOutput(stage, outputVar.type, outputVar.name);
+            mergeContext->registerOutput(stage, outputVar.type, outputVar.name, outputVar.flat);
     }
 
     for (auto it = mergeContext->m_inOutVars.cbegin(), end = mergeContext->m_inOutVars.cend(); it != end; ++it) {
@@ -426,11 +517,14 @@ void QSSGProgramGenerator::registerShaderMetaDataFromSource(QSSGShaderResourceMe
     }
 }
 
-QSSGRef<QSSGRhiShaderPipeline> QSSGProgramGenerator::compileGeneratedRhiShader(const QByteArray &inMaterialInfoString,
-                                                                               const QSSGShaderFeatures &inFeatureSet,
-                                                                               QSSGShaderLibraryManager &shaderLibraryManager,
-                                                                               QSSGShaderCache &theCache,
-                                                                               QSSGRhiShaderPipeline::StageFlags stageFlags)
+QSSGRhiShaderPipelinePtr QSSGProgramGenerator::compileGeneratedRhiShader(const QByteArray &inMaterialInfoString,
+                                                                         const QSSGShaderFeatures &inFeatureSet,
+                                                                         QSSGShaderLibraryManager &shaderLibraryManager,
+                                                                         QSSGShaderCache &theCache,
+                                                                         QSSGRhiShaderPipeline::StageFlags stageFlags,
+                                                                         const QSSGUserShaderAugmentation &shaderAugmentation,
+                                                                         int viewCount,
+                                                                         bool perTargetCompilation)
 {
     // No stages enabled
     if (((quint32)m_enabledStages) == 0) {
@@ -439,6 +533,7 @@ QSSGRef<QSSGRhiShaderPipeline> QSSGProgramGenerator::compileGeneratedRhiShader(c
     }
 
     QSSGShaderResourceMergeContext mergeContext;
+    mergeContext.viewCount = viewCount;
 
     for (quint32 stageIdx = 0; stageIdx < static_cast<quint32>(QSSGShaderGeneratorStage::StageCount); ++stageIdx) {
         QSSGShaderGeneratorStage stageName = static_cast<QSSGShaderGeneratorStage>(1 << stageIdx);
@@ -465,6 +560,8 @@ QSSGRef<QSSGRhiShaderPipeline> QSSGProgramGenerator::compileGeneratedRhiShader(c
         }
     }
 
+    QSSGShaderResourceMergeContext::setAdditionalBufferAmount(0);
+
     // qDebug("VERTEX:\n%s\n\n", m_vs.m_finalBuilder.constData());
     // qDebug("FRAGMENT:\n%s\n\n", m_fs.m_finalBuilder.constData());
 
@@ -472,7 +569,10 @@ QSSGRef<QSSGRhiShaderPipeline> QSSGProgramGenerator::compileGeneratedRhiShader(c
                                    m_vs.m_finalBuilder,
                                    m_fs.m_finalBuilder,
                                    inFeatureSet,
-                                   stageFlags);
+                                   stageFlags,
+                                   shaderAugmentation,
+                                   viewCount,
+                                   perTargetCompilation);
 }
 
 QSSGVertexShaderGenerator::QSSGVertexShaderGenerator()
@@ -486,6 +586,7 @@ QSSGFragmentShaderGenerator::QSSGFragmentShaderGenerator()
 void QSSGFragmentShaderGenerator::addShaderIncomingMap()
 {
     addShaderItemMap(ShaderItemType::Input, m_incoming);
+    addShaderItemMap(ShaderItemType::Input, m_flatIncoming, ShaderItemMapFlag::Flat);
     addShaderPass2Marker(ShaderItemType::Input);
 }
 
