@@ -1,5 +1,7 @@
 // Copyright (C) 2023 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+// Qt-Security score:significant reason:default
+
 
 #ifndef QSSGLIGHTMAPPER_H
 #define QSSGLIGHTMAPPER_H
@@ -14,6 +16,7 @@
 //
 
 #include <QtQuick3DRuntimeRender/qtquick3druntimerenderexports.h>
+#include <QString>
 
 QT_BEGIN_NAMESPACE
 
@@ -27,6 +30,9 @@ struct QSSGLightmapperOptions
     int indirectLightWorkgroupSize = 32;
     int indirectLightBounces = 3;
     float indirectLightFactor = 1.0f;
+    QString source = QStringLiteral("lightmaps.bin");
+    float sigma = 8.f;
+    float texelsPerUnit = 1.f;
 };
 
 QT_END_NAMESPACE
