@@ -1,6 +1,8 @@
 // Copyright (C) 2008-2012 NVIDIA Corporation.
 // Copyright (C) 2019 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+// Qt-Security score:significant reason:default
+
 
 #include "qssgrenderbuffermanager_p.h"
 
@@ -138,7 +140,7 @@ static QPair<QSSGMesh::Mesh, QString> loadFromLightmapFile(const QString &lightm
         return retVal;
 
     if (const auto io = QSSGLightmapLoader::open(lightmapPath)) {
-        const QVariantMap metadata = io->readMetadata(lightmapKey);
+        const QVariantMap metadata = io->readMap(lightmapKey, QSSGLightmapIODataTag::Metadata);
         if (metadata.isEmpty())
             return retVal;
 
@@ -222,6 +224,8 @@ QSSGRenderImageTexture QSSGBufferManager::loadRenderImage(const QSSGRenderImage
                 theImage = qsgImageMap.insert(qsgTexture, ImageData());
             theImage.value().renderImageTexture.m_texture = qsgTexture->rhiTexture();
             theImage.value().renderImageTexture.m_flags.setHasTransparency(qsgTexture->hasAlphaChannel());
+            // If a texture from Qt Quick has an alpha channel, it is always pre-multiplied
+            theImage.value().renderImageTexture.m_flags.setPreMultipliedAlpha(qsgTexture->hasAlphaChannel());
             theImage.value().usageCounts[currentLayer]++;
             result = theImage.value().renderImageTexture;
             // inMipMode is ignored completely when sourcing the texture from a
@@ -471,6 +475,14 @@ QRhiTexture::Format QSSGBufferManager::toRhiFormat(const QSSGRenderTextureFormat
     case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_12x12:
         return QRhiTexture::ASTC_12x12;
 
+    case QSSGRenderTextureFormat::Depth16:
+        return QRhiTexture::D16;
+    case QSSGRenderTextureFormat::Depth24:
+        return QRhiTexture::D24;
+    case QSSGRenderTextureFormat::Depth32:
+        return QRhiTexture::D32F;
+    case QSSGRenderTextureFormat::Depth24Stencil8:
+        return QRhiTexture::D24S8;
 
     case QSSGRenderTextureFormat::SRGB8A8:
         return QRhiTexture::RGBA8; // Note: user must keep track of color space manually
@@ -1510,6 +1522,21 @@ void QSSGBufferManager::releaseExtensionResult(const QSSGRenderExtension &rext)
     renderExtensionTexture.remove(&rext);
 }
 
+void QSSGBufferManager::releaseUserRenderPass(const QSSGRenderUserPass &upass)
+{
+    // Go through the user pass managers and tell them to unschedule this pass
+    std::vector<QSSGUserRenderPassManagerWeakPtr> activeManagers;
+    activeManagers.reserve(userRenderPassManagers.size());
+    for (const auto &wmptr : std::as_const(userRenderPassManagers)) {
+        if (const auto &mng = wmptr.lock()) {
+            mng->unscheduleUserPass(&upass);
+            activeManagers.push_back(wmptr);
+        }
+    }
+
+    userRenderPassManagers = std::move(activeManagers);
+}
+
 void QSSGBufferManager::releaseMesh(const QSSGRenderPath &inSourcePath)
 {
     QMutexLocker meshMutexLocker(&meshBufferMutex);
@@ -2250,6 +2277,11 @@ void QSSGBufferManager::setCurrentlyLightmapBaking(bool value)
     currentlyLightmapBaking = value;
 }
 
+void QSSGBufferManager::registerUserRenderPassManager(const QSSGUserRenderPassManagerPtr &userPassManager)
+{
+    userRenderPassManagers.push_back(userPassManager);
+}
+
 size_t qHash(const QSSGBufferManager::CustomImageCacheKey &k, size_t seed) noexcept
 {
     // NOTE: The data pointer should never be null, as null data pointers shouldn't be inserted into
